;; ebizorist/theme.jl
;; $Id: $
;
;    E B I Z O R I S T  version 0.12
;
;    W A R N I N G: Dont edit theme.jl directly. Edit new.jl and run make(1).
;

;    E B I Z O R I S T :   A theme for sawfish window manager.
;    Copyright (C) 2002 Ito Yo-zi
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.





;(require 'sawfish.wm.util.recolor-image)
;(require 'rep.data.tables)

(defgroup ebizorist "Ebizorist"
  :group appearance)

(defcustom ebizorist:experimental nil
  "Enable experimental features. (use under expert user-level is recommended)"
  :type boolean
  :group (appearance ebizorist)
  :user-level intermediate
  :after-set after-setting-frame-option)

(defcustom ebizorist:debug nil
  "Print debug messages. (for experts)"
  :type boolean
  :group (appearance ebizorist)
  :user-level expert
  :after-set after-setting-frame-option)


(defcustom ebizorist:title-font
  (get-font "-*-*-*-i-*-*-14-*-*-*-*-*-*-*")
  "Used for window title normally."
  :type font
  :group (appearance ebizorist)
  :user-level novice
  :after-set after-setting-frame-option)

(defcustom ebizorist:compressed-title-font
  (get-font "-*-*-*-*-*-*-12-*-*-*-*-*-*-*")
  "Used for longer window title. set smaller font than normal one."
  :type font
  :group (appearance ebizorist)
  :user-level novice
  :after-set after-setting-frame-option)

(defcustom ebizorist:navigation-mode 'default
  "Navigations of mouse buttons on edges/corners: \\w"
  :type (choice
	 (default "move-window for dragging.")
	 (standard "same as other themes.")
	 )
  :group (appearance ebizorist)
  :after-set after-setting-frame-option)





(let*
    (
     ;; ebizorist:debug is not set yet.
     ;; Through theme.jl, debug mode is true.
     (ebizorist:debug t)
     ;;; coller pettern table. 
     ;;; its according to subdirectory containing png images. ;;;
     (pixmap-dirs (list "green" "orange" "red" "white"
			"blue" "yellow" "cyan" "magenda"))


     ;;;;;;;;;;;; general functions ;;;;;;;;;;;;


     (translate-list (lambda (cnt ilist body)
		       (eval `(let ((dirs ',ilist)
				    (,cnt nil)
				    (outlist nil))
				(while dirs
				  (setq ,cnt (car dirs))
				  (setq outlist (cons ,body outlist))
				  (setq dirs (cdr dirs)))
				(reverse outlist)))))

     (debug (lambda x
	      (if ebizorist:debug
		  (message (eval (append '(concat " ")
					 (translate-list 'e x
							 '(format nil " %s" e))
					 nil))))))


;(eval (append `(format nil "debug: %s")
;					 x))))))

     ;;;;;;;;;;;; image generator functions ;;;;;;;;;;;;

     (make-image-list (lambda (dir prefix)
			(debug "loading " dir "/" prefix)
			(list (make-image (concat dir "/" prefix "U.png"))
			      (make-image (concat dir "/" prefix "F.png")))))

     (add-hilight-to-image-list (lambda (imlist)
				  (list (nth 0 imlist)
					(nth 1 imlist)
					(nth 0 imlist))))

     (make-image-bundle
      (lambda (prefix)
	(translate-list 'count pixmap-dirs
			`(,make-image-list count ,prefix))))


     (flip-image-list (lambda (imlist)
			  (translate-list
			   'image imlist
			   `(flip-image-horizontally (copy-image image)))))

     (flip-image-bundle (lambda (bundle)
			  (translate-list
			   'ilist bundle
			   `(,translate-list
			     'image ilist
			     `(flip-image-horizontally (copy-image image))))))

     (hilightate-image-bundle (lambda (bundle)
				(translate-list
				 'ilist bundle
				 `(,add-hilight-to-image-list ilist))))


     ;;;;;;;;;;;;;;;;; image displayer functions ;;;;;;;;;;;;;;;;;

     (choose-color-index (lambda (w bundle)
		       (% (% (* (window-id w) 32767) 9699689)
			  (length pixmap-dirs))))

     (get-image-hook (lambda (bundle)
		       (lambda (w)
			 (debug "run hook" '(a 'aaaa))
			 (nth (choose-color-index w bundle) bundle))))


     (title-font (lambda (w)
;		   (message (window-name w))
		   (if (<
			(text-width (window-name w) ebizorist:title-font)
			(- (car (window-dimensions w)) 56))
		       ebizorist:title-font
		     ebizorist:compressed-title-font
		     )))

     (button-font (lambda (w)
;		   (message (font-name ebizori:title-font))
		   ebizorist:title-font))


;     (bottom-border-cursor (lambda (w)
;			     (case ebizorist:navigation-mode
;			       ((default) 'draped_box)
;			       ((standard) 'bottom_side)
;			       )))

     (right-border-cursor (lambda (w)
			     (case ebizorist:navigation-mode
			       ((default) 'draped_box)
			       ((standard) 'right_side)
			       )))

     (left-border-cursor (lambda (w)
			     (case ebizorist:navigation-mode
			       ((default) 'draped_box)
			       ((standard) 'left_side)
			       )))

;     (bottom-border-button1-move (lambda ()
;				   (case ebizorist:navigation-mode
;				     ((default)
;				      (move-window-interactively))
;				     ((standard)
;				      (resize-window-interactively)))))


;     (bottom-border-keymap (bind-keys (make-keymap)
;				      "Button1-Click2"
;				      (lambda () (message "click2!"))
;;				       "Button1-Move"
;;				       (lambda () (message "move!"))
;;				      "Button1-Move" 'resize-window-interactively
;				      "Button1-Move" 'move-window-interactively
;;				      "Button2-Move" 'move-window-interactively
;				      "Button3-Click" 'raise-lower-window
;				      ))


     (get-bottom-border (lambda ()
			  (case ebizorist:navigation-mode
			    ((default) 'default-bottom-border)
			    ((standard) 'bottom-border)
			    )))

     (get-right-border (lambda ()
			  (case ebizorist:navigation-mode
			    ((default) 'default-right-border)
			    ((standard) 'right-border)
			    )))

     (get-left-border (lambda ()
			  (case ebizorist:navigation-mode
			    ((default) 'default-left-border)
			    ((standard) 'left-border)
			    )))

     (get-bottom-left-corner (lambda ()
			       (case ebizorist:navigation-mode
				 ((default) 'default-bottom-left-corner)
				 ((standard) 'bottom-left-corner)
				 )))

     (get-bottom-right-corner (lambda ()
			       (case ebizorist:navigation-mode
				 ((default) 'default-bottom-right-corner)
				 ((standard) 'bottom-right-corner)
				 )))

     (get-top-left-corner (lambda ()
			       (case ebizorist:navigation-mode
				 ((default) 'default-top-left-corner)
				 ((standard) 'top-left-corner)
				 )))

     (get-top-right-corner (lambda ()
			       (case ebizorist:navigation-mode
				 ((default) 'default-top-right-corner)
				 ((standard) 'top-right-corner)
				 )))

     ;;;;;;; image definitions (normal/shaped frame) ;;;;;;;;;;

     (bottom-image-list (make-image-bundle "bottom"))
     (top-image-list (make-image-bundle "top"))
     (right-image-list (make-image-bundle "right"))
     (right-bottom-image-list (make-image-bundle "cornerRB"))
     (right-top-image-list (make-image-bundle "cornerRT"))
     (button-image-list (make-image-bundle "button"))

     (bottom-images (get-image-hook bottom-image-list))
     (top-images (get-image-hook top-image-list))
     (right-images (get-image-hook right-image-list))
     (left-images (get-image-hook (flip-image-bundle right-image-list)))
     (right-bottom-images (get-image-hook right-bottom-image-list))
     (right-top-images (get-image-hook right-top-image-list))
     (left-bottom-images (get-image-hook (flip-image-bundle right-bottom-image-list)))
     (left-top-images (get-image-hook (flip-image-bundle right-top-image-list)))

     (button-images (get-image-hook (hilightate-image-bundle button-image-list)))


     ;;;;;;; image definitions (transient frame) ;;;;;;;;;;

     (transient-bottom-images (make-image-list "transient" "trans-bottom"))
     (transient-top-images (make-image-list "transient" "trans-top"))
     (transient-right-images (make-image-list "transient" "trans-right"))
     (transient-left-images (flip-image-list transient-right-images))
     (transient-right-bottom-images (make-image-list "transient" "trans-cornerRB"))
     (transient-left-bottom-images (flip-image-list transient-right-bottom-images))
     (transient-right-top-images (make-image-list "transient" "trans-cornerRT"))
     (transient-left-top-images (flip-image-list transient-right-top-images))




     ;;;;;;; frame definitions ;;;;;;;;;;

     (frame (lambda (w type)
	      (debug "frame:" (window-id w) (window-name w) type)
	      (let ((f `(
		       (
			(left-edge . -16)
			(bottom-edge . -4)
			(top-edge . -4)
			(background . ,(left-images w))
;		 (cursor . ,left-border-cursor)
;		 (keymap . ,bottom-border-keymap)
			(class . ,(get-left-border))
			)
		       (
			(background . ,(right-images w))
			(top-edge . -4)
			(bottom-edge . -4)
			(right-edge . -16)
;		 (cursor . ,right-border-cursor)
;		 (keymap . ,bottom-border-keymap)
			(class . ,(get-right-border))
			)
		       (
			(background . ,(bottom-images w))
			(right-edge . -4)
			(left-edge . -4)
			(bottom-edge . -16)
;		 (cursor . ,bottom-border-cursor)
;		 (keymap . ,bottom-border-keymap)
;		 (class . bottom-border)
			(class . ,(get-bottom-border))
			)
		       (
			(right-edge . -32)
			(top-edge . -32)
			(background . ,(right-top-images w))
;		 (class . top-right-corner)
			(class . ,(get-top-right-corner))
			)
		       (
			(top-edge . -32)
			(left-edge . -32)
			(background . ,(left-top-images w))
;		 (class . top-left-corner)
			(class . ,(get-top-left-corner))
			)
		       (
			(background . ,(right-bottom-images w))
			(right-edge . -32)
			(bottom-edge . -32)
;		 (class . bottom-right-corner)
			(class . ,(get-bottom-right-corner))
			)
		       (
			(bottom-edge . -32)
			(left-edge . -32)
			(background . ,(left-bottom-images w))
;		 (class . bottom-left-corner)
			(class . ,(get-bottom-left-corner))
			)
		       (
			(right-edge . 28)
			(top-edge . -18)
			(left-edge . 28)
			(font . ,title-font)
			(y-justify . center)
			(x-justify . center)
			(background . ,(top-images w))
			(foreground . "black")
			(text . ,window-name)
			(class . title)
			)
;          (class . top-border))
		       (
			(top-edge . -18)
			(right-edge . -12)
			(width . 20)
			(background . ,(button-images w))
			(foreground . "black")
			(font . ,button-font)
			(y-justify . center)
			(x-justify . center)
			(text . "X")
			(class . close-button)
			)
		       (
			(top-edge . -18)
			(left-edge . -12)
			(width . 20)
			(background . ,(button-images w))
			(foreground . "black")
			(font . ,button-font)
			(y-justify . center)
			(x-justify . center)
			(text . "X")
			(class . close-button)
			)
		       (
			(top-edge . -18)
			(left-edge . 8)
			(width . 20)
			(background . ,(button-images w))
			(foreground . "black")
			(font . ,button-font)
			(y-justify . center)
			(x-justify . center)
			(text . "?")
			(class . menu-button)
			)
		       (
			(top-edge . -18)
			(right-edge . 8)
			(width . 20)
			(background . ,(button-images w))
			(foreground . "black")
			(font . ,button-font)
			(y-justify . center)
			(x-justify . center)
			(text . "-")
			(class . iconify-button)
			)
		       )))
		(debug "frame end:" (window-id w))
		f)))



     ;;;;;;; shaped frame definitions ;;;;;;;;;;

     (shaped-frame `(
	      (
	       (right-edge . -32)
	       (top-edge . -32)
	       (background . ,right-top-images)
	       (class . top-right-corner))
	      (
	       (top-edge . -32)
	       (left-edge . -32)
	       (background . ,left-top-images)
	       (class . top-left-corner))


	      (
	       (right-edge . 28)
	       (top-edge . -18)
	       (left-edge . 28)
	       (font . ,title-font)
	       (y-justify . center)
	       (x-justify . center)
	       (background . ,top-images)
	       (foreground . "black")
	       (text . ,window-name)
	       (class . title))
;          (class . top-border))
	      (
	       (top-edge . -18)
	       (right-edge . -12)
	       (width . 20)
	       (background . ,button-images)
	       (foreground . "black")
	       (font . ,button-font)
	       (y-justify . center)
	       (x-justify . center)
	       (text . "X")
	       (class . close-button))
	      (
	       (top-edge . -18)
	       (left-edge . -12)
	       (width . 20)
	       (background . ,button-images)
	       (foreground . "black")
	       (font . ,button-font)
	       (y-justify . center)
	       (x-justify . center)
	       (text . "X")
	       (class . close-button))
	      (
	       (top-edge . -18)
	       (left-edge . 8)
	       (width . 20)
	       (background . ,button-images)
	       (foreground . "black")
	       (font . ,button-font)
	       (y-justify . center)
	       (x-justify . center)
	       (text . "?")
	       (class . menu-button))
	      (
	       (top-edge . -18)
	       (right-edge . 8)
	       (width . 20)
	       (background . ,button-images)
	       (foreground . "black")
	       (font . ,button-font)
	       (y-justify . center)
	       (x-justify . center)
	       (text . "-")
	       (class . iconify-button))

	      ))


     ;;;;;;;;;;; transient frame definitions ;;;;;;;;;;;

     (transient-frame `(
	      (
	       (left-edge . -4)
	       (bottom-edge . -1)
	       (top-edge . -1)
	       (background . ,transient-left-images)
	       (class . left-border))
	      (
	       (background . ,transient-right-images)
	       (top-edge . -1)
	       (bottom-edge . -1)
	       (right-edge . -4)
	       (class . right-border))
	      (
	       (background . ,transient-bottom-images)
	       (right-edge . -1)
	       (left-edge . -1)
	       (bottom-edge . -4)
	       (class . bottom-border)
	       )
	      (
	       (right-edge . -8)
	       (top-edge . -8)
	       (background . ,transient-right-top-images)
	       (class . top-right-corner))
	      (
	       (top-edge . -8)
	       (left-edge . -8)
	       (background . ,transient-left-top-images)
	       (class . top-left-corner))
	      (
	       (background . ,transient-right-bottom-images)
	       (right-edge . -8)
	       (bottom-edge . -8)
	       (class . bottom-right-corner))
	      (
	       (bottom-edge . -8)
	       (left-edge . -8)
	       (background . ,transient-left-bottom-images)
	       (class . -bottom-left-corner))


	      (
	       (right-edge . -1)
	       (top-edge . -5)
	       (left-edge . -1)
;	       (font . ,title-font)
;	       (y-justify . center)
;	       (x-justify . center)
	       (background . ,transient-top-images)
;	       (text . ,window-name)
	       (class . title))
	      ))


     )


  ;;;;;;;;;;;;;;; class definitions ;;;;;;;;;;;;;;;;

  (def-frame-class default-bottom-border
    `(
      (cursor . ,(case ebizorist:navigation-mode
		   ((default) 'draped_box)
		   ((standard) 'bottom_side)))
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button4-Click"
			    (lambda () (debug "button4!"))
			    "Button1-Move" 'move-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))

  (def-frame-class default-right-border
    `(
      (cursor . ,(case ebizorist:navigation-mode
		   ((default) 'draped_box)
		   ((standard) 'right_side)))
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button1-Move" 'move-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))

  (def-frame-class default-left-border
    `(
      (cursor . ,(case ebizorist:navigation-mode
		   ((default) 'draped_box)
		   ((standard) 'left_side)))
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button1-Move" 'move-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))

  (def-frame-class default-bottom-left-corner
    `(
      (cursor . bottom_left_corner)
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button1-Move" 'resize-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))

  (def-frame-class default-bottom-right-corner
    `(
      (cursor . bottom_right_corner)
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button1-Move" 'resize-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))

  (def-frame-class default-top-left-corner
    `(
      (cursor . top_left_corner)
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button1-Move" 'resize-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))

  (def-frame-class default-top-right-corner
    `(
      (cursor . top_right_corner)
      (keymap . ,(bind-keys (make-keymap)
			    "Button1-Click2"
			    (lambda () (debug "click2!"))
			    "Button1-Move" 'resize-window-interactively
			    "Button3-Click" 'raise-lower-window
			    ))
      ))




  ;;;;;;;;;;;;;;; add frame ;;;;;;;;;;;;;;;;

  (add-frame-style 'ebizorist
		   (lambda (w type)
;		     (message (format nil "type: %s num: %s" type (window-id w)))
		     (case type
		       ((default) (frame w type))
		       ((transient) transient-frame)
		       ((shaped) shaped-frame)
		       ((shaped-transient) transient-frame))))

  ;; repainting frame is needed when title string had been changed.
  (add-hook 'property-notify-hook
	    (lambda (w atom type)
	      (if (eq atom 'WM_NAME)
		  (reframe-window w)
		)))

;  (add-hook 'enter-flipper-hook
;	    (lambda (sym)
;	      (message (format nil "enter-flip: " sym))))
;  (add-hook 'leave-flipper-hook
;	    (lambda (sym)
;	      (message (format nil "leave-flip: " sym))))
  (debug "----------------------------------")
  )


